<?php

namespace WPDesk\ShopMagic\ActionExecution;

use WPDesk\ShopMagic\Action\Action;
use WPDesk\ShopMagic\Automation\Automation;
use WPDesk\ShopMagic\AutomationOutcome\OutcomeReposistory;
use WPDesk\ShopMagic\DataSharing\DataLayer;
use WPDesk\ShopMagic\Event\Event;
use WPDesk\ShopMagic\Event\DeferredStateCheck\SupportsDeferredCheck;
use WPDesk\ShopMagic\Exception\ActionDisabledAfterStatusRecheckException;
use WPDesk\ShopMagic\LoggerFactory;
use WPDesk\ShopMagic\Placeholder\PlaceholderFactory;
use WPDesk\ShopMagic\Placeholder\PlaceholderProcessor;

/**
 * Executes action NOW!. Do not use any queue od other async tools.
 *
 * @package WPDesk\ShopMagic\ActionExecution
 */
class ExecuteNow implements ExecutionStrategy {
	/** @var PlaceholderFactory */
	private $placeholder_factory;

	public function __construct( PlaceholderFactory $placeholder_factory ) {
		$this->placeholder_factory = $placeholder_factory;
	}

	/**
	 * @inheritDoc
	 */
	public function execute( Automation $automation, Event $event, Action $action, $action_index, $unique_id ) {
		if ( $event instanceof SupportsDeferredCheck && ! $event->is_event_still_valid() ) {
			LoggerFactory::get_logger()->notice( "Deferred check. Event: {$event->get_name()}. Automation: {$automation->get_id()}. Order linked to Event has changed status again and is no longer consistent with this event.",
				[
					'automation'   => $automation,
					'event'        => $event,
					'action'       => $action,
					'action_index' => $action_index,
					'unique_id'    => $unique_id
				] );

			throw new ActionDisabledAfterStatusRecheckException( __( 'Order linked to Event has changed status again and is no longer consistent with this event', 'shopmagic-for-woocommerce' ) );
		}
		$data_layer = new DataLayer( $event, [ $automation, $event, $action ] );
		$processor  = new PlaceholderProcessor( $this->placeholder_factory, $data_layer );

		$action->set_placeholder_processor( $processor );
		$action->set_provided_data( $data_layer->get_provided_data() );

		try {
			$result = $action->execute( $automation, $event );
			$this->save_outcome( $unique_id, $result );
		} catch ( \Throwable $e ) {
			$this->save_outcome( $unique_id, false, $e->getMessage() );
			throw $e;
		} catch ( \Exception $e ) { // TODO: can be removed in PHP7
			$this->save_outcome( $unique_id, false, $e->getMessage() );
			throw $e;
		}

		return $result;
	}

	/**
	 * @param string $unique_id
	 * @param bool $result
	 * @param string|null $note
	 */
	private function save_outcome( $unique_id, $result, $note = null ) {
		global $wpdb;
		$outcome_repository = new OutcomeReposistory( $wpdb );
		$outcome_repository->finish_outcome( $unique_id, $result, $note );
	}
}
