<?php

namespace WPDesk\ShopMagic\ActionExecution;

use WPDesk\ShopMagic\Action\Action;
use WPDesk\ShopMagic\Automation\Automation;
use WPDesk\ShopMagic\Automation\AutomationFactory;
use WPDesk\ShopMagic\Event\Event;
use WPDesk\ShopMagic\Placeholder\PlaceholderFactory;
use \WC_Queue_Interface;

/**
 * Responsible for creating strategies how to execute an action. This particular factory is using Queue.
 *
 * @package WPDesk\ShopMagic\ActionExecution
 */
class QueueExecutionStrategyFactory implements ExecutionStrategyFactory {
	const ACTION_EXECUTE_FROM_QUEUE = 'shopmagic/core/queue/execute';
	const FILTER_AVOID_QUEUE = 'shopmagic/core/queue/avoid_queue';

	const QUEUE_GROUP = 'shopmagic-automation';

	/** @var PlaceholderFactory */
	private $placeholder_factory;

	/** @var AutomationFactory */
	private $automation_factory;

	/** @var WC_Queue_Interface */
	private $queue_client;

	public function __construct( PlaceholderFactory $placeholder_factory ) {
		$this->placeholder_factory = $placeholder_factory;
	}

	/**
	 * @inheritDoc
	 */
	public function set_automation_factory( AutomationFactory $automation_factory ) {
		$this->automation_factory = $automation_factory;
	}

	/**
	 * Whe is delayed action it has parameters with info about delay.
	 *
	 * @param Action $action
	 *
	 * @return bool
	 */
	private function is_delayed_action( Action $action ) {
		try {
			return $action->get_fields_data()->has( '_action_delayed' ) && $action->get_fields_data()->get( '_action_delayed' ) === 'on'; // see: shopmagic-delayed-actions plugin
		} catch (\Exception $e) {
			// TODO: better persistence library
			return false;
		}
	}

	/**
	 * @param Action $action
	 *
	 * @return ExecutionStrategy
	 */
	private function create_delayed_executor( Action $action ) {
		$delay = (int) $action->get_fields_data()->get( '_action_delayed_offset_time' ); // see: shopmagic-delayed-actions plugin

		return new ExecuteQueueScheduled( $this->queue_client, time() + $delay );
	}

	/**
	 * @inheritDoc
	 */
	public function create_strategy( Automation $automation, Event $event, Action $action, $action_index ) {
		// TODO: refactor in 3.0 in sync with Delayed Action plugin. Should use fields.
		if ( $this->is_delayed_action( $action ) ) {
			return $this->create_delayed_executor( $action );
		}

		$avoid_queue = apply_filters( self::FILTER_AVOID_QUEUE, false, $automation, $event, $action, $action_index );
		if ( $avoid_queue ) {
			return new ExecuteNow( $this->placeholder_factory );
		}

		return new ExecuteQueueNow( $this->queue_client );
	}

	/**
	 * Use hooks to catch deferred works from queue, refresh them with data and execute using ExecuteNow strategy.
	 *
	 * @return void
	 */
	public function initialize(\WC_Queue_Interface $queue) {
		add_action( 'woocommerce_init', function () {
			$this->queue_client = \WC_Queue::instance();
		} );

		add_action( self::ACTION_EXECUTE_FROM_QUEUE, [ $this, 'run_action' ], 10, 5 );
	}

	/**
	 * Run action from queue
	 *
	 * @internal
	 * @noinspection NullPointerExceptionInspection False positive in $automation->get_event.
	 */
	public function run_action( $automation_serialized, $event_serialized, $action_serialized, $action_index, $unique_id = null ) {
		if ($unique_id === null) {
			$unique_id = uniqid('fallback_', true);
		}
		$run = function () use ( $automation_serialized, $event_serialized, $action_index, $unique_id ) {
			$automation = $this->automation_factory->create_automation( $automation_serialized['id'] ); // see: Automation::jsonSerialize

			$event = $automation->get_event();
			$event->set_from_json( $event_serialized ); // see: Event::jsonSerialize

			$action = $automation->get_action( $action_index );

			$executor = new ExecuteNow( $this->placeholder_factory );
			$executor->execute( $automation, $event, $action, $action_index, $unique_id );
		};

		if ( DOING_CRON ) {
			$run();
		} else {
			if ( did_action( 'wp_loaded' ) ) {
				$run();
			} else {
				add_action( 'wp_loaded', $run );
			}
		}
	}
}
