<?php

namespace WPDesk\ShopMagic\Admin\Automation;

use ShopMagicVendor\WPDesk\View\Renderer\SimplePhpRenderer;
use ShopMagicVendor\WPDesk\View\Resolver\DirResolver;
use WPDesk\ShopMagic\Automation\AutomationFactory;
use WPDesk\ShopMagic\Automation\AutomationPersistence;
use WPDesk\ShopMagic\Event\ManualEvent;

class ManualActionsConfirmPage {
	const SLUG = 'manual-action-confirm';

	/** @var AutomationFactory */
	private $automation_factory;

	/**
	 * @param AutomationFactory $automation_factory
	 */
	public function __construct( AutomationFactory $automation_factory ) {
		$this->automation_factory = $automation_factory;
	}

	public function hooks() {
		add_action( 'admin_menu', function () {
			add_submenu_page(
				'edit.php?post_type=shopmagic_automation',
				__( 'Manual action', 'shopmagic-for-woocommerce' ),
				__( 'Manual action', 'shopmagic-for-woocommerce' ),
				'manage_options',
				self::SLUG,
				[ $this, 'render_manual_actions' ]
			);
		} );

		add_action( 'admin_head', static function() {
			remove_submenu_page( 'edit.php?post_type=shopmagic_automation', self::SLUG );
		} );

		/**
		 * Filters out custom fields from being duplicated by Duplicate Post plugin in addition to the defaults.
		 *
		 * @param array $meta_excludelist The default exclusion list, based on the “Do not copy these fields” setting, plus some other field names.
		 *
		 * @return array The custom fields to exclude.
		 */
		add_filter( 'duplicate_post_excludelist_filter', static function ( $meta_excludelist ) {
			return array_merge( $meta_excludelist, [ 'manual_started', 'manual_started_by' ] );
		} );
	}

	/**
	 * Loops for all orders by trying to preserve as much memory as possible using paginationns + generators.
	 *
	 * @return \Generator|\WC_Order[]
	 */
	private function generate_orders() {
		$page = 1;
		do {
			$orders_paged = wc_get_orders( [
				'paginate' => true,
				'page'     => $page ++,
			] );
			foreach ( $orders_paged->orders as $order ) {
				if ($order instanceof \WC_Order) {
					yield $order;
				}
			}
		} while ( $page <= $orders_paged->max_num_pages );
	}

	/**
	 * @param ManualEvent $event
	 *
	 * @return \Generator|\WC_Order[]
	 */
	private function generate_matched_orders( ManualEvent $event ) {
		foreach ( $this->generate_orders() as $order ) {
			$event->set_provided_data( [ \WC_Order::class => $order, \WP_User::class => $order->get_user() ] );
			if ( $event->is_filter_passed() ) {
				yield $order;
			}
		}
	}

	/**
	 * @param ManualEvent $event
	 */
	private function trigger_for_matched_orders( ManualEvent $event ) {
		foreach ( $this->generate_orders() as $order ) {
			$event->set_provided_data( [ \WC_Order::class => $order, \WP_User::class => $order->get_user() ] );
			$event->trigger();
		}
	}

	/**
	 * @internal action
	 */
	public function render_manual_actions() {
		if ( ! isset( $_GET['post'] ) || ! current_user_can( 'manage_options' ) ) {
			return;
		}
		$automation = $this->automation_factory->create_automation( $_GET['post'] );
		$event      = $automation->get_event();
		if ( ! $event instanceof ManualEvent ) {
			return;
		}

		$persistence      = new AutomationPersistence( $automation->get_id() );
		$never_run_before = !$persistence->is_manual_action_ever_started();
		if ( $never_run_before && isset( $_POST['run'] ) ) {
			$persistence->set_manual_action_started();;
			$this->trigger_for_matched_orders( $event );
			$never_run_before = false;
		}

		$renderer = new SimplePhpRenderer( new DirResolver( __DIR__ . DIRECTORY_SEPARATOR . 'templates' ) );
		if ( $never_run_before ) {
			echo $renderer->render( 'manual_actions_confirm', [
				'automation'               => $automation,
				'matched_orders_generator' => $this->generate_matched_orders( $event )
			] );
		} else {
			echo $renderer->render( 'manual_actions_done', [
				'automation' => $automation
			] );
		}
	}
}
