<?php

namespace WPDesk\ShopMagic;

use ShopMagicVendor\WPDesk\Notice;
use ShopMagicVendor\WPDesk\ShowDecision\PostTypeStrategy;
use ShopMagicVendor\WPDesk_Plugin_Info;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Psr\Log\NullLogger;
use ShopMagicVendor\WPDesk\Logger\WPDeskLoggerFactory;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\AbstractPlugin;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\HookableCollection;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\HookableParent;
use WPDesk\ShopMagic\ActionExecution\QueueExecutionStrategyFactory;
use WPDesk\ShopMagic\ActionExecution\ExecutionStrategyFactory;
use WPDesk\ShopMagic\Admin\Admin;
use WPDesk\ShopMagic\Action\ActionFactory;
use WPDesk\ShopMagic\Admin\RateNotice\TwoWeeksNotice;
use WPDesk\ShopMagic\Admin\Settings\GeneralSettings;
use WPDesk\ShopMagic\Admin\RateNotice\RateNotices;
use WPDesk\ShopMagic\Admin\Settings\Settings;
use WPDesk\ShopMagic\Automation\AutomationFactory;
use WPDesk\ShopMagic\CommunicationList\CommunicationListPostType;
use WPDesk\ShopMagic\Automation\AutomationPostType;
use WPDesk\ShopMagic\Beacon\Beacon;
use WPDesk\ShopMagic\Database\DatabaseSchema;
use WPDesk\ShopMagic\Event\Builtin\OrderPending;
use WPDesk\ShopMagic\Event\EventFactory;
use WPDesk\ShopMagic\Filter\FilterFactory;
use WPDesk\ShopMagic\Frontend;
use WPDesk\ShopMagic\Optin;
use WPDesk\ShopMagic\Placeholder\PlaceholderFactory;
use WPDesk\ShopMagic\Tracker;

/**
 * Main plugin class. The most important flow decisions are made here.
 *
 * @package WPDesk\ShopMagic
 */
class Plugin extends AbstractPlugin implements LoggerAwareInterface, HookableCollection {
	use LoggerAwareTrait;
	use HookableParent;

	/**
	 * Most info about plugin internals.
	 *
	 * @var WPDesk_Plugin_Info
	 */
	protected $plugin_info;

	/** @var EventFactory */
	private $event_factory;

	/** @var FilterFactory */
	private $filter_factory;

	/** @var ActionFactory */
	private $action_factory;

	/** @var PlaceholderFactory */
	private $placeholder_factory;

	/** @var FormIntegration */
	private $form_integration;

	/** @var AutomationFactory */
	private $automation_factory;

	/** @var ExecutionStrategyFactory */
	private $execution_factory;

	/** @var DatabaseSchema */
	private $db_schema;

	public function __construct( WPDesk_Plugin_Info $plugin_info ) {
		class_alias( self::class, 'ShopMagic' );
		$this->logger = new NullLogger();

		/** @noinspection PhpParamsInspection */
		parent::__construct( $plugin_info );

		$this->forms_integration();
		$this->event_factory       = apply_filters( 'shopmagic/core/factory/event', new EventFactory() );
		$this->filter_factory      = apply_filters( 'shopmagic/core/factory/filter', new \WPDesk\ShopMagic\Filter\FilterFactory() );
		$this->action_factory      = apply_filters( 'shopmagic/core/factory/action', new ActionFactory() );
		$this->placeholder_factory = apply_filters( 'shopmagic/core/factory/placeholder', new PlaceholderFactory() );
		$this->form_integration    = new FormIntegration();
		$this->execution_factory   = apply_filters( 'shopmagic/core/factory/execution',
			new QueueExecutionStrategyFactory( $this->placeholder_factory )
		);

		$this->automation_factory = new AutomationFactory(
			$this->event_factory,
			$this->action_factory,
			$this->filter_factory,
			$this->execution_factory
		);

		$this->execution_factory->set_automation_factory( $this->automation_factory );

		define( 'SHOPMAGIC_BASE_FILE', 'deprecated' );
		define( 'SHOPMAGIC_PLUGIN_URL', $plugin_info->get_plugin_url() . '/' ); // deprecated

		$this->db_schema = new DatabaseSchema( $plugin_info->get_plugin_file_name() );
		$this->db_schema->register_activation_hook();
	}

	/**
	 * Integrate with WordPress and with other plugins using action/filter system.
	 *
	 * @return void
	 */
	public function hooks() {
		parent::hooks();

		require_once __DIR__ . '/Admin/Welcome/Popups.php';

		if ( $this->db_schema->is_old_database() ) {
			$this->db_schema->install();
		}

		add_action( 'plugins_loaded', function () {
			$this->setLogger( $this->is_debug_mode() ? ( new WPDeskLoggerFactory() )->createWPDeskLogger() : new NullLogger() );
			LoggerFactory::set_logger( $this->logger );

			$queue = \WC_Queue::instance();
			/** @noinspection NullPointerExceptionInspection */
			$this->execution_factory->initialize( $queue );
			/** @noinspection NullPointerExceptionInspection */
			OrderPending::initialize_pending_on_created_check( $queue );
		} );

		add_action( 'woocommerce_init', function () {
			( new Notice\AjaxHandler( $this->get_plugin_url() . '/vendor_prefixed/wpdesk/wp-notice/assets' ) )->hooks();
			( new AutomationPostType( $this->event_factory ) )->hooks();
			( new CommunicationListPostType() )->hooks();

			( new Tracker\DeactivationTracker( $this->plugin_info->get_plugin_file_name() ) )->hooks();
			( new Tracker\TrackerNotices() )->hooks();
			( new Tracker\UsageDataTracker( $this->plugin_info->get_plugin_file_name() ) )->hooks();

			( new Frontend\ListsOnCheckout() )->hooks();
			( new Frontend\ListsOnAccount() )->hooks();
			( new OptIn\NewUserBehaviour() )->hooks();

			// should be called BEFORE initialize_active_woocommerce_automations so the dependant plugins could attach themselves
			do_action( 'shopmagic/core/initialized', $this->plugin_info->get_version() );

			$this->action_factory->initialize_actions_with_test_data( $this->placeholder_factory, LoggerFactory::get_logger() );

			// required woocommerce_init for WC methods to fully works
			$automations = $this->automation_factory->initialize_active_woocommerce_automations();

			if ( is_admin() ) {
				( new RateNotices(
					[ new TwoWeeksNotice( $this->plugin_url . '/assets', $automations, new PostTypeStrategy( AutomationPostType::TYPE ) ) ]
				) )->hooks();
			}
		} );

		add_action( 'shopmagic/core/settings/ready', function () {
			if ( is_admin() && GeneralSettings::get_option( 'disable_beacon', 'no' ) !== 'yes' ) {
				( new Beacon(
					'6057086f-4b25-4e12-8735-fbc556d2dc01',
					new PostTypeStrategy( AutomationPostType::TYPE ),
					$this->get_plugin_assets_url()
				) )->hooks();
			}
		} );

		if ( is_admin() ) {
			( new Admin(
				$this->plugin_info->get_plugin_url(),
				$this->event_factory,
				$this->filter_factory,
				$this->action_factory,
				$this->placeholder_factory,
				$this->automation_factory,
				$this->form_integration
			) )->hooks();
		}

		$this->hooks_on_hookable_objects();
	}

	/**
	 * Returns true when debug mode is on.
	 *
	 * @return bool
	 */
	private function is_debug_mode() {
		$helper_options = get_option( 'wpdesk_helper_options', [] );

		return isset( $helper_options['debug_log'] ) && '1' === $helper_options['debug_log'];
	}

	/**
	 * It's a good idea that Form fields have a local namespace to create more stable env for other plugins.
	 */
	private function forms_integration() {
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\CheckboxField::class, \WPDesk\ShopMagic\FormField\Field\CheckboxField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\Header::class, \WPDesk\ShopMagic\FormField\Field\Header::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\HiddenField::class, \WPDesk\ShopMagic\FormField\Field\HiddenField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\InputTextField::class, \WPDesk\ShopMagic\FormField\Field\InputTextField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\NoOnceField::class, \WPDesk\ShopMagic\FormField\Field\NoOnceField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\NoValueField::class, \WPDesk\ShopMagic\FormField\Field\NoValueField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\Paragraph::class, \WPDesk\ShopMagic\FormField\Field\Paragraph::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\ProductSelect::class, \WPDesk\ShopMagic\FormField\Field\ProductSelect::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\RadioField::class, \WPDesk\ShopMagic\FormField\Field\RadioField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\SelectField::class, \WPDesk\ShopMagic\FormField\Field\SelectField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\SubmitField::class, \WPDesk\ShopMagic\FormField\Field\SubmitField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\TextAreaField::class, \WPDesk\ShopMagic\FormField\Field\TextAreaField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\WooSelect::class, \WPDesk\ShopMagic\FormField\Field\WooSelect::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\WyswigField::class, \WPDesk\ShopMagic\FormField\Field\WyswigField::class );

		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\NoValueField::class, \WPDesk\ShopMagic\FormField\NoValueField::class );
		class_alias( \ShopMagicVendor\WPDesk\Forms\Field\BasicField::class, \WPDesk\ShopMagic\FormField\BasicField::class );
	}

	/**
	 * Quick links on plugins page.
	 *
	 * @param array $links .
	 *
	 * @return array
	 */
	public function links_filter( $links ) {
		$plugin_links = [];

		$plugin_links[] = '<a href="' . Settings::get_url() . '">' . __( 'Settings',
				'shopmagic-for-woocommerce' ) . '</a>';
		$plugin_links[] = '<a href="https://shopmagic.app/docs/" target="_blank">' . __( 'Docs',
				'shopmagic-for-woocommerce' ) . '</a>';
		$plugin_links[] = '<a href="https://wordpress.org/support/plugin/shopmagic-for-woocommerce/" target="_blank">' . __( 'Support',
				'shopmagic-for-woocommerce' ) . '</a>';

		if ( ! shopmagic_is_pro_active() ) {
			$plugin_links[] = '<a href="https://shopmagic.app/pricing/?utm_source=user-site&utm_medium=quick-link&utm_campaign=shopmagic-upgrade" target="_blank" style="color:#d64e07;font-weight:bold;">' . __( 'Upgrade',
					'shopmagic-for-woocommerce' ) . '</a>';
		}

		return array_merge( $plugin_links, $links );
	}
}
